package de.dhbwka.java.exercise.classes;

import java.util.Random;
import java.util.Scanner;

public class MasterMind {
  private static final int COMBINATION_LENGTH = 5;
  private static final int MAX_ATTEMPTS = 20;
  private static final char[] COLORS = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H' };
  private String secretCombination;

  public MasterMind() {
    this.secretCombination = generateSecretCombination();
  }

  private String generateSecretCombination() {
    Random random = new Random();
    StringBuilder combination = new StringBuilder(COMBINATION_LENGTH);
    for (int i = 0; i < COMBINATION_LENGTH; i++) {
      combination.append(COLORS[random.nextInt(COLORS.length)]);
    }
    return combination.toString();
  }

  public void play() {
    Scanner scanner = new Scanner(System.in);
    int attempts = 0;
    boolean won = false;

    System.out.println("Welcome to MasterMind!");
    System.out.println("Try to guess the combination of 5 letters (A to H). You have " + MAX_ATTEMPTS + " attempts.");

    while (attempts < MAX_ATTEMPTS && !won) {
      System.out.print("Enter your guess: ");
      String guess = scanner.nextLine().toUpperCase();

      if (guess.length() != COMBINATION_LENGTH) {
        System.out.println("Invalid input. Please enter exactly " + COMBINATION_LENGTH + " letters.");
        continue;
      }

      attempts++;
      int[] result = evaluateGuess(guess);
      System.out.println("Correct positions: " + result[0] + ", Correct letters in wrong positions: " + result[1]);

      if (result[0] == COMBINATION_LENGTH) {
        won = true;
        System.out.println("Congratulations! You've guessed the correct combination.");
      }
    }

    if (!won) {
      System.out.println("Sorry, you've used all your attempts. The correct combination was: " + secretCombination);
    }

    scanner.close();
  }

  private int[] evaluateGuess(String guess) {
    int correctPositions = 0;
    int correctLettersWrongPositions = 0;
    boolean[] secretUsed = new boolean[COMBINATION_LENGTH];
    boolean[] guessUsed = new boolean[COMBINATION_LENGTH];

    // First pass: check for correct positions
    for (int i = 0; i < COMBINATION_LENGTH; i++) {
      if (guess.charAt(i) == secretCombination.charAt(i)) {
        correctPositions++;
        secretUsed[i] = true;
        guessUsed[i] = true;
      }
    }

    // Second pass: check for correct letters in wrong positions
    for (int i = 0; i < COMBINATION_LENGTH; i++) {
      if (!guessUsed[i]) {
        for (int j = 0; j < COMBINATION_LENGTH; j++) {
          if (!secretUsed[j] && guess.charAt(i) == secretCombination.charAt(j)) {
            correctLettersWrongPositions++;
            secretUsed[j] = true;
            break;
          }
        }
      }
    }

    return new int[] { correctPositions, correctLettersWrongPositions };
  }

}
