package de.dhbwka.java.exercise.classes;

public class Polynomial {
  private double[] factors;

  public Polynomial(double[] factors) {
    if (factors == null || factors.length != 3) {
      throw new IllegalArgumentException("Factors must not be null and must have a length of 3");
    }
    this.factors = factors;
  }

  public double[] getFactors() {
    return factors;
  }

  public void setFactors(double[] factors) {
    this.factors = factors;
  }

  @Override
  public String toString() {
    String result = "";

    if (factors[0] != 0) {
      if (factors[0] < 0) {
        result += "-";
      }
      if (factors[0] != 1 && factors[0] != -1) {
        result += Math.abs(factors[0]);
      }
      result += "x^2";
    }

    if (factors[1] != 0) {
      if (factors[1] < 0) {
        result += " - ";
      } else if (factors[0] != 0) {
        result += " + ";
      }
      if (factors[1] != 1 && factors[1] != -1) {
        result += Math.abs(factors[1]);
      }
      result += "x";
    }

    if (factors[2] != 0) {
      if (factors[2] < 0) {
        result += " - ";
      } else if (factors[0] != 0 || factors[1] != 0) {
        result += " + ";
      }
      result += Math.abs(factors[2]);
    }

    return result;
  }

  public double evaluate(double x) {
    return factors[0] * x * x + factors[1] * x + factors[2];
  }

  public Polynomial add(Polynomial p) {
    return new Polynomial(
        new double[] { factors[0] + p.factors[0], factors[1] + p.factors[1], factors[2] + p.factors[2] });
  }

  public Polynomial sub(Polynomial p) {
    return new Polynomial(
        new double[] { factors[0] - p.factors[0], factors[1] - p.factors[1], factors[2] - p.factors[2] });
  }

  public Polynomial scalarMul(double scalar) {
    return new Polynomial(new double[] { factors[0] * scalar, factors[1] * scalar, factors[2] * scalar });
  }

  public double[] findRoots() {
    double discriminant = factors[1] * factors[1] - 4 * factors[0] * factors[2];
    if (factors[0] == 0 && factors[1] == 0) {
      return new double[0];
    } else if (factors[0] == 0) {
      return new double[] { -factors[2] / factors[1] };
    } else if (discriminant < 0) {
      return new double[0];
    } else if (discriminant == 0) {
      return new double[] { -factors[1] / (2 * factors[0]) };
    } else {
      double root1 = (-factors[1] + Math.sqrt(discriminant)) / (2 * factors[0]);
      double root2 = (-factors[1] - Math.sqrt(discriminant)) / (2 * factors[0]);
      return new double[] { root1, root2 };
    }
  }
}
