package de.dhbwka.java.exercise.classes.periodic;

public class PeriodicTable {
  private Element[] elements = new Element[118];

  public void addElement(Element e) {
    if (elements[e.getOrdinal()] == null) {
      elements[e.getOrdinal()] = e;
    }
  }

  public Element getElement(int ordinal) {
    return elements[ordinal];
  }

  public boolean hasElement(Element e) {
    return elements[e.getOrdinal()] != null;
  }

  public Element[] getMetals() {
    int count = 0;
    for (Element e : elements) {
      if (e instanceof Metal) {
        count++;
      }
    }
    Element[] metals = new Element[count];
    count = 0;
    for (Element e : elements) {
      if (e instanceof Metal) {
        metals[count] = e;
        count++;
      }
    }
    return metals;
  }

  public static void main(String[] args) {
    PeriodicTable pt = new PeriodicTable();
    // H
    pt.addElement(new Element("Wasserstoff", "H", 1, 'K', 2, true));
    // He
    pt.addElement(new Element("Helium", "He", 2, 'K', 2, true));
    // Na
    pt.addElement(new Metal("Natrium", "Na", 11, 'L', 0, true, false, 21.0 * Math.pow(10, 6)));
    // Fe
    pt.addElement(new Metal("Eisen", "Fe", 26, 'M', 0, true, false, 10.02 * Math.pow(10, 6)));
    // Ge
    pt.addElement(new Metal("Germanium", "Ge", 32, 'N', 0, true, true, 1.45));
    // Br
    pt.addElement(new Element("Brom", "Br", 35, 'N', 1, true));
    // Te
    pt.addElement(new Metal("Tellur", "Te", 52, 'O', 0, true, true, 0.005));
    // Au
    pt.addElement(new Metal("Gold", "Au", 79, 'P', 0, true, false, 44.0 * Math.pow(10, 6)));

    for (Element e : pt.getMetals()) {
      System.out.println(e);
    }

    System.out.println(pt.getElement(79));
  }
}

class Element {
  private String name;
  private String symbol;
  private int ordinal;
  private char shell;
  private int phase;
  private boolean isMainGroup;

  public Element(String name, String symbol, int ordinal, char shell, int phase, boolean isMainGroup) {
    this.name = name;
    this.symbol = symbol;
    this.ordinal = ordinal;
    this.shell = shell;
    this.phase = phase;
    this.isMainGroup = isMainGroup;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getSymbol() {
    return symbol;
  }

  public void setSymbol(String symbol) {
    this.symbol = symbol;
  }

  public int getOrdinal() {
    return ordinal;
  }

  public void setOrdinal(int ordinal) {
    this.ordinal = ordinal;
  }

  public char getShell() {
    return shell;
  }

  public void setShell(char shell) {
    this.shell = shell;
  }

  public int getPhase() {
    return phase;
  }

  public void setPhase(int phase) {
    this.phase = phase;
  }

  public boolean isMainGroup() {
    return isMainGroup;
  }

  public void setMainGroup(boolean isMainGroup) {
    this.isMainGroup = isMainGroup;
  }

  @Override
  public String toString() {
    return name + " (" + symbol + ", " + ordinal + ")" + " Schale: " + shell + ", Phase: "
        + (phase == 0 ? "fest" : (phase == 1 ? "flüssig" : "gasförmig"))
        + (isMainGroup ? ", Hauptgruppe" : ", Nebengruppe");
  }

  public boolean equals(Element e) {
    return this.ordinal == e.ordinal;
  }
}

class Metal extends Element {
  private boolean metalloid;
  private double conductivity;

  public Metal(String name, String symbol, int ordinal, char shell, int phase, boolean isMainGroup, boolean metalloid,
      double conductivity) {
    super(name, symbol, ordinal, shell, phase, isMainGroup);
    this.metalloid = metalloid;
    this.conductivity = conductivity;
  }

  public boolean isMetalloid() {
    return metalloid;
  }

  public void setMetalloid(boolean metalloid) {
    this.metalloid = metalloid;
  }

  public double getConductivity() {
    return conductivity;
  }

  public void setConductivity(double conductivity) {
    this.conductivity = conductivity;
  }

  @Override
  public String toString() {
    return super.toString() + (metalloid ? ", Halbleiter" : "") + ", Leitfähigkeit: " + conductivity;
  }

}