package de.dhbwka.java.exercise.threads;

import java.awt.Color;
import java.awt.Graphics;

import javax.swing.JComponent;
import javax.swing.JFrame;

class LightPhase {
   private String name;
   private boolean red;
   private boolean yellow;
   private boolean green;
   private int duration;
   private int next;

   public LightPhase(String name, boolean red, boolean yellow, boolean green, int duration, int next) {
      this.name = name;
      this.red = red;
      this.yellow = yellow;
      this.green = green;
      this.duration = duration;
      this.next = next;
   }

   public LightPhase getNext() {
      return LightPhases[next];
   }

   public String getName() {
      return name;
   }

   public boolean isRed() {
      return red;
   }

   public boolean isYellow() {
      return yellow;
   }

   public boolean isGreen() {
      return green;
   }

   public int getDuration() {
      return duration;
   }

   private final static LightPhase LightPhases[] = {
         new LightPhase("Red", true, false, false, 500, 1),
         new LightPhase("Red+Yellow", true, true, false, 200, 2),
         new LightPhase("Green", false, false, true, 500, 3),
         new LightPhase("Yellow", false, true, false, 200, 0)
   };

   public static LightPhase[] getLightPhases() {
      return LightPhases;
   }
}

/**
 * Part of lectures on 'Programming in Java'. Baden-Wuerttemberg
 * Cooperative State University.
 *
 * (C) 2016-2018 by W. Geiger, T. Schlachter, C. Schmitt, W. Suess
 *
 * @author DHBW lecturer
 * @version 1.1-STATICLIGHT
 */
@SuppressWarnings("serial")
public class TrafficLight extends JComponent {
   private final static long DELAY = 500;

   LightPhase currentPhase = LightPhase.getLightPhases()[0];

   public TrafficLight() {
      Thread t = new Thread() {
         @Override
         public void run() {
            while (true) {
               repaint();
               try {
                  Thread.sleep(currentPhase.getDuration());
               } catch (InterruptedException e) {
                  e.printStackTrace();
               }
               currentPhase = currentPhase.getNext();
            }
         }
      };
      t.setDaemon(true);
      t.start();
   }

   @Override
   public void paintComponent(Graphics g) {
      super.paintComponent(g);

      // traffic light box
      g.setColor(Color.BLACK);
      g.fillRect(10, 10, 80, 195);

      // 3 x empty light
      g.setColor(Color.WHITE);
      g.fillOval(23, 23, 54, 54);
      g.fillOval(23, 83, 54, 54);
      g.fillOval(23, 143, 54, 54);

      /*
       * ====================================================
       * ====================================================
       * SET THE CORRECT VALUES FOR THE FOLLOWING 3 VARIABLES
       * ====================================================
       * ====================================================
       */
      boolean redOn = currentPhase.isRed();
      boolean yellowOn = currentPhase.isYellow();
      boolean greenOn = currentPhase.isGreen();

      // draw colored lights, if active
      if (redOn) {
         g.setColor(Color.RED);
         g.fillOval(25, 25, 50, 50);
      }

      if (yellowOn) {
         g.setColor(Color.YELLOW);
         g.fillOval(25, 85, 50, 50);
      }
      if (greenOn) {
         g.setColor(Color.GREEN);
         g.fillOval(25, 145, 50, 50);
      }
   }

   public static void main(String[] args) {
      // Create frame and add TrafficLight component
      JFrame f = new JFrame("Traffic Light");
      f.add(new TrafficLight());
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setSize(100, 260);
      f.setVisible(true);
   }

}
