package de.dhbwka.java.exercise.threads.buffer;

import java.util.LinkedList;

public class MyBuffer {
  LinkedList<Integer> buffer;
  static final int MAX_SIZE = 10;

  public MyBuffer() {
    buffer = new LinkedList<>();
  }

  public synchronized void put(int value) throws InterruptedException {
    while (buffer.size() >= MAX_SIZE) {
      wait(); // Wait until space is available
    }
    buffer.add(value);
    System.out.println("Produced: " + value);
    notifyAll(); // Notify consumers that an item has been added
  }

  public synchronized int get() throws InterruptedException {
    while (buffer.isEmpty()) {
      wait(); // Wait until an item is available
    }
    int value = buffer.removeFirst();
    System.out.println("Consumed: " + value);
    notifyAll(); // Notify producers that space is available
    return value;
  }

  public synchronized int size() {
    return buffer.size();
  }
}

class Producer extends Thread {
  private MyBuffer buffer;

  public Producer(MyBuffer buffer) {
    this.buffer = buffer;
  }

  @Override
  public void run() {
    try {
      for (int i = 0; i < 20; i++) {
        buffer.put(i);
        Thread.sleep((long) (Math.random() * 1000));
        System.out.println("Produced: " + i);
        System.out.println("Buffer size: " + buffer.size());
      }
    } catch (InterruptedException e) {
      Thread.currentThread().interrupt();
    }
  }
}

class Consumer extends Thread {
  private MyBuffer buffer;

  public Consumer(MyBuffer buffer) {
    this.buffer = buffer;
  }

  @Override
  public void run() {
    try {
      for (int i = 0; i < 20; i++) {
        int value = buffer.get();
        Thread.sleep((long) (Math.random() * 1000));
        System.out.println("Processed: " + value);
        System.out.println("Buffer size: " + buffer.size());
      }
    } catch (InterruptedException e) {
      Thread.currentThread().interrupt();
    }
  }
}

class Runner {
  public static void main(String[] args) {
    MyBuffer buffer = new MyBuffer();
    Producer producer = new Producer(buffer);
    Consumer consumer = new Consumer(buffer);

    producer.start();
    consumer.start();

    try {
      producer.join();
      consumer.join();
    } catch (InterruptedException e) {
      Thread.currentThread().interrupt();
    }
  }
}
