package de.dhbwka.java.exercise.threads.search;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class SearchEngine {
  private static final int MAX_THREADS = 3;
  private List<PageLoader> running_loaders = new ArrayList<>();
  private static final String file = "src/de/dhbwka/java/exercise/threads/search/uri.txt";

  public SearchEngine() throws InterruptedException {
    try (BufferedReader br = new BufferedReader(new FileReader(file))) {
      Iterator<String> lines = br.lines().iterator();
      while (running_loaders.size() > 0 || lines.hasNext()) {
        if (running_loaders.size() < MAX_THREADS && lines.hasNext()) {
          String line = lines.next();
          PageLoader pl = new PageLoader(line, "UTF-8");
          Thread plThread = new Thread(pl);
          System.out.println("Loading " + line);
          plThread.start();
          running_loaders.add(pl);
        }

        List<PageLoader> toRemove = new ArrayList<>();
        for (PageLoader pl : this.running_loaders) {
          if (pl != null && pl.pageLoaded()) {
            System.out.println(
                "Page " + pl.getUrl() + " loaded: " + pl.getPageContent().replaceAll("\n", "##").substring(0, 40));
            toRemove.add(pl);
          }
        }
        running_loaders.removeAll(toRemove);

        Thread.sleep(100);
      }
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  public static void main(String[] args) throws InterruptedException {
    new SearchEngine();
  }
}

class PageLoader implements Runnable {
  private String content;

  public synchronized boolean pageLoaded() {
    return content != null;
  }

  public synchronized String getPageContent() {
    if (!pageLoaded())
      throw new Error("Page not loaded");
    return content;
  }

  public static String getStringContentFromUrl(String url, String encoding) {
    StringBuilder buffer = new StringBuilder();
    String line = null;
    try (BufferedReader br = new BufferedReader(
        new InputStreamReader(
            new URI(url).toURL().openStream(), encoding))) {
      while ((line = br.readLine()) != null) {
        buffer.append(line).append(System.lineSeparator());
      }
    } catch (Exception ex) {
    }
    return buffer.toString();
  }

  private String url;
  private String encoding;

  public String getUrl() {
    return url;
  }

  public PageLoader(String url, String encoding) {
    this.url = url;
    this.encoding = encoding;
  }

  @Override
  public void run() {
    this.content = getStringContentFromUrl(url, encoding);
  }
}
