package de.dhbwka.java.exercise.ui;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.*;

public class CurrencyCalculator extends JFrame {
  public static void main(String[] args) {
    new CurrencyCalculator();
  }

  public CurrencyCalculator() {
    initComponents();
    addEventListeners();
  }

  private static void addComponent(Container cont, Component c,
      int x, int y, int width, int height, double weightx, double weighty) {
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = width;
    gbc.gridheight = height;
    gbc.weightx = weightx;
    gbc.weighty = weighty;
    cont.add(c, gbc);
  }

  JTextField amountField = new JTextField();
  JButton cancelButton = new JButton("Cancel");
  JButton convertButton = new JButton("Convert");

  String[] currencies = { "EUR", "USD", "GBP", "CHF", "DM" };
  double[] rates = { 1.0, 0.90054095, 1.1896249, 1.0711047, 0.51129 };
  
  JComboBox<String> currencyBoxFrom = new JComboBox<>(currencies);
  JComboBox<String> currencyBoxTo = new JComboBox<>(currencies);

  private void initComponents() {
    setTitle("Currency Calculator");
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setLocationRelativeTo(null);
    setSize(400, 300);

    JPanel panel = new JPanel();
    panel.setLayout(new GridBagLayout());

    addComponent(panel, amountField, 0, 0, 4, 1, 1.0, 0.0);
    addComponent(panel, currencyBoxFrom, 0, 1, 1, 1, 0.0, 0.0);
    addComponent(panel, currencyBoxTo, 1, 1, 1, 1, 0.0, 0.0);
    addComponent(panel, convertButton, 2, 1, 1, 1, 0.0, 0.0);
    addComponent(panel, cancelButton, 3, 1, 1, 1, 0.0, 0.0);

    add(panel);
    pack();
    setVisible(true);
  }

  private void addEventListeners() {
    convertButton.addActionListener(e -> {
      String from = (String) currencyBoxFrom.getSelectedItem();
      String to = (String) currencyBoxTo.getSelectedItem();
      convertCurrency(from, to);
    });
    cancelButton.addActionListener(e -> {
      this.dispose();
    });
  };

  private void convertCurrency(String from, String to) {
    // get the amount from the text field
    String amountText = amountField.getText();
    double amount = 0;
    try {
      amount = Double.parseDouble(amountText);
    } catch (NumberFormatException e) {
      JOptionPane.showMessageDialog(this, "Invalid amount", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }
    // convert the amount to intermediate currency (EUR)
    double intermediateAmount = 0;
    int fromIndex = -1;
    for (int i = 0; i < currencies.length; i++) {
      if (currencies[i].equals(from)) {
        fromIndex = i;
        break;
      }
    }
    if (fromIndex == -1) {
      JOptionPane.showMessageDialog(this, "Invalid currency", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }
    // convert the amount to EUR
    intermediateAmount = amount * rates[fromIndex];

    // convert the intermediate amount to the target currency
    double targetAmount = 0;
    int toIndex = -1;
    for (int i = 0; i < currencies.length; i++) {
      if (currencies[i].equals(to)) {
        toIndex = i;
        break;
      }
    }
    if (toIndex == -1) {
      JOptionPane.showMessageDialog(this, "Invalid currency", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }
    // convert the intermediate amount to the target currency
    targetAmount = intermediateAmount / rates[toIndex];

    // show the result in a message dialog
    String message = String.format("Converted amount: %.2f %s", targetAmount, to);
    JOptionPane.showMessageDialog(this, message, "Result",
        JOptionPane.INFORMATION_MESSAGE);
    // clear the text field
    amountField.setText("");
    // set focus to the text field
    amountField.requestFocus();
  }
}
