package de.dhbwka.java.exercise.ui;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import javax.swing.*;

public class TextFrame extends JFrame {
  public static void main(String[] args) {
    if (args.length != 3 || args[0].equals("-h")) {
      System.err.println("Usage: java TextFrame <filename> <width> <height>");
      System.exit(1);
    }
    try {
      int width = Integer.parseInt(args[1]);
      int height = Integer.parseInt(args[2]);
      new TextFrame(width, height, args[0]);
    } catch (NumberFormatException e) {
      System.err.println("Width and height must be integers.");
      System.exit(1);
    }
  }

  public TextFrame(int width, int height, String fileName) {
    // Load the text file
    String text = loadTextFile(fileName);
    if (text == null) {
      System.err.println("Error loading file: " + fileName);
      return;
    }
    // Create the text area
    JTextArea textArea = new JTextArea(text);
    textArea.setLineWrap(true);
    textArea.setWrapStyleWord(true);
    JScrollPane scrollPane = new JScrollPane(textArea);
    scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

    // Create the frame
    this.setTitle("TextFrame");
    this.setSize(width, height);
    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    this.setLocationRelativeTo(null);
    this.add(scrollPane);
    this.setVisible(true);
  }

  private String loadTextFile(String fileName) {
    StringBuilder text = new StringBuilder();
    try (BufferedReader reader = new BufferedReader(new FileReader(fileName))) {
      String line;
      while ((line = reader.readLine()) != null) {
        text.append(line).append("\n");
      }
    } catch (IOException e) {
      e.printStackTrace();
      return null;
    }
    return text.toString();
  }
}
